/////////////////////////////////////////////////////////////////////////////
// Name:        ServerData.cpp
// Purpose:     Store the server parameters here
// Author(s):   Atul, Ashesh
// Created:     2008-07-08
// RCS-ID:      $Id: ServerData.cpp,v 1.9 2008/08/29 12:00:54 asheshv Exp $
// Copyright:   (c) EnterpriseDB
/////////////////////////////////////////////////////////////////////////////

#include "Server.h"
#include "WSUtils.h"

#ifdef WIN32
#define STRICMP(str1, str2) _stricmp(str1, str2)
#else
#define STRICMP(str1, str2) strcasecmp(str1, str2)
#endif //WIN32

const wxString ServerConfigData::ms_parameters[DYNATUNE_CONFIG_LEN] =
{
    wxT("max_connections"),
    wxT("shared_buffers"),
    wxT("work_mem"),
    wxT("maintenance_work_mem"),
    wxT("max_fsm_pages"),
    wxT("max_fsm_relations"),
    wxT("wal_buffers"),
    wxT("checkpoint_segments"),
    wxT("random_page_cost"),
    wxT("effective_cache_size"),
    wxT("autovacuum"),
    wxT("autovacuum_naptime"),
    wxT("autovacuum_vacuum_threshold"),
    wxT("autovacuum_analyze_threshold"),
    wxT("autovacuum_vacuum_scale_factor"),
    wxT("autovacuum_analyze_scale_factor"),
    wxT("stats_row_level")
};

ConfigValue ServerConfigData::ms_ConfigFlags[DYNATUNE_CONFIG_LEN] = {CV_NOT_CHANGED};

/**
*  Name: ClearConfigFlags
*  Desc: Clear the configuration flags
*        - Use this function, before making first time call to 
*          UpdateConfigBuffer, while updating the postgresql.conf
*          (configuration file).
*        - It will be useful to maintain the list of updated
*          parameters for that particular configuration files.
*          
*/
void Server::ClearConfigFlags()
{
    for(int i =0; i < DYNATUNE_CONFIG_LEN; i++)
    {
        ServerConfigData::ms_ConfigFlags[i] = CV_NOT_CHANGED;
    }
}


static wxString g_strCommentStr  = wxT("#");
static size_t      g_iCommentLen = (size_t)1;

/**
*  Name: UpdateConfigBuffer
*  Desc: Update the buffer, if it matches following criteria:
*        1. Format of the buffer matches to "parameter = value #some_comment"
*        2. buffer contains one of the parameter from the parameter list 
*           (ms_parameters)
*        3. The configuration has valid value for that particular parameter
*        Resulted buffer will in the following format:
*           "parameter = new_value #some_comment"
*/
wxString Server::UpdateConfigBuffer(wxString buffer, 
                                    int& matched_parameter_index,
                                    wxString& original_value,
                                    wxString& new_value,
                                    bool& isCommented)
{
    wxString comments;

    // Initialize the parameters to default values
    original_value.Empty();
    isCommented = false;
    matched_parameter_index = -1;

    /**
    * Itreate through all the parameter list, to check if 
    * this buffer contains one of the parameter in the following
    * format:
    *    parameter = value
    * We can have zero or more no of whitespace in between this
    * Keywords
    */
    wxString restOfTheString;
    wxString tempBuffer = buffer.Trim(false);
    if(tempBuffer.StartsWith(g_strCommentStr)) {
        isCommented = true;
        tempBuffer = wxString(tempBuffer.substr(g_iCommentLen, tempBuffer.Length())).Trim(false);
    }
    for(int i=0; i < DYNATUNE_CONFIG_LEN; i++)
    {
        if(tempBuffer.StartsWith(ServerConfigData::ms_parameters[i]))
        {
            restOfTheString
                = tempBuffer.substr(
                                  ServerConfigData::ms_parameters[i].length());
            if(restOfTheString.Trim(false).StartsWith(wxT("=")))
            {
                restOfTheString = wxString(restOfTheString.Trim(false).substr(
                                           1,restOfTheString.length())).Trim(false);
                matched_parameter_index = i;
                break;
            }
        }
    }

    // No parameter matched with the current buffer
    if(matched_parameter_index == -1)
        return buffer;

    // Check if this parameter found multiple times
    // If found second time, return an empty string
    if(ServerConfigData::ms_ConfigFlags[matched_parameter_index] == CV_CHANGED)
    {
        return wxT("");
    }
    /**
    *  If value for the matched parameter is empty, then
    *  we will not modify the values it.
    */
    if(m_cfgData.m_parameter_value[matched_parameter_index].IsEmpty())
    {
        ServerConfigData::ms_ConfigFlags[matched_parameter_index] = CV_CHANGED;
        return buffer;
    }

    ServerConfigData::ms_ConfigFlags[matched_parameter_index]
                               = isCommented ? CV_COMMENTED : CV_CHANGED;
    
    //Fetching original value
    original_value = restOfTheString.BeforeFirst(' ').BeforeFirst('\t').BeforeFirst('#');
    
    //Fetching comments
    comments = restOfTheString.substr(original_value.length(), restOfTheString.length());

    new_value = m_cfgData.m_parameter_value[matched_parameter_index];

    return 
        ServerConfigData::ms_parameters[matched_parameter_index] + wxT(" = ") +
        new_value + comments;
}

/**
*  Name: SetValue
*  Desc: Set the value for the particular parameter, if it is found 
*        in the list of parameter list. (ms_parameters)
*/
void ServerConfigData::SetValue(wxString &prop, wxString &value) {
    for(int i=0; i < DYNATUNE_CONFIG_LEN; i++)
    {
        if(STRICMP(
            (const char *)ms_parameters[i].mb_str(wxConvUTF8),
            (const char *) prop.mb_str(wxConvUTF8)) == 0)
        {
            m_parameter_value[i] = value;
            break;
        }
    }
}

wxString Server::GetNextNonUpdatedConfiguration()
{
    for(int i=0; i < DYNATUNE_CONFIG_LEN; i++)
    {
        if(ServerConfigData::ms_ConfigFlags[i] == CV_NOT_CHANGED)
        {
            ServerConfigData::ms_ConfigFlags[i] = CV_CHANGED;
            if(ServerConfigData::ms_parameters[i] == wxT("stats_row_level"))
                return wxEmptyString;
            return ServerConfigData::ms_parameters[i] +
                wxT(" = ") +
                m_cfgData.m_parameter_value[i];
        }
    }
    return wxEmptyString;
}

